module CagnutPipelineDraw
  module Pipeline
    class Recalibration
      attr_accessor :gatk, :picard, :samtools

      def initialize job_name
        @job_name = job_name
        @config = Cagnut::Configuration.config
        @gatk = ::CagnutGatk::Util.new @config
        @picard = ::CagnutPicard::Util.new @config
        @samtools = ::CagnutSamtools::Util.new
      end

      def run filename = nil, order=1
        job_name, filename, order = picard.markdup picard_markup_dirs, order, @job_name, filename
        order = samtools.do_flag_stat samtools_do_flag_stat_dirs, order, job_name, filename
        order = gatk.count_read gatk_count_read_dirs, order, job_name, filename
        job_name, filename, order = gatk.recal gatk_recal_dirs, order, job_name, filename
        order = picard.picard_qc_metrics picard_picard_qc_metrics_dirs, order, job_name, filename
        [job_name, filename, order]
      end

      private

      def picard_markup_dirs
        {
          input: @config['sample']['bam'],
          output: @config['sample']['bam'],
          tmp_dir: @config['sample']['tmp']
        }
      end

      def samtools_do_flag_stat_dirs
        {
          input: @config['sample']['bam'],
          output: @config['sample']['stat']
        }
      end

      def gatk_count_read_dirs
        {
          input: @config['sample']['bam'],
          output: @config['sample']['stat']
        }
      end

      def gatk_recal_dirs
        {
          input: @config['sample']['bam'],
          output: @config['sample']['bam'],
          contrast: @config['sample']['bam']
        }
      end

      def picard_picard_qc_metrics_dirs
        {
          input: @config['sample']['bam'],
          output: @config['sample']['stat'],
          tmp_dir: @config['sample']['tmp']
        }
      end

    end
  end
end
